@@ -19,6 +19,8 @@ module Text.Hamlet
     , HtmlUrl
     , hamlet
     , hamletFile
+    , hamletFileReload
+    , ihamletFileReload
     , xhamlet
     , xhamletFile
       -- * I18N Hamlet
@@ -40,11 +42,20 @@ module Text.Hamlet
     , ihamletRules
     , htmlRules
     , CloseStyle (..)
+      -- * Used by generated code
+    , condH
+    , maybeH
+    , asHtmlUrl
+    , attrsToHtml
     ) where
 
 import Text.Shakespeare.Base
 import Text.Hamlet.Parse
+#if MIN_VERSION_template_haskell(2,9,0)
+import Language.Haskell.TH.Syntax hiding (Module)
+#else
 import Language.Haskell.TH.Syntax
+#endif
 import Language.Haskell.TH.Quote
 import Data.Char (isUpper, isDigit)
 import Data.Maybe (fromMaybe)
@@ -54,10 +65,18 @@ import Text.Blaze.Html (Html, toHtml)
 import Text.Blaze.Internal (preEscapedText)
 import qualified Data.Foldable as F
 import Control.Monad (mplus)
-import Data.Monoid (mempty, mappend)
+import Data.Monoid (mempty, mappend, mconcat)
 import Control.Arrow ((***))
 import Data.List (intercalate)
 
+import Data.IORef
+import qualified Data.Map as M
+import System.IO.Unsafe (unsafePerformIO)
+import Filesystem (getModified)
+import Data.Time (UTCTime)
+import Filesystem.Path.CurrentOS (decodeString)
+import Text.Blaze.Html (preEscapedToHtml)
+
 -- | Convert some value to a list of attribute pairs.
 class ToAttributes a where
     toAttributes :: a -> [(Text, Text)]
@@ -224,10 +243,6 @@ docToExp env hr scope (DocCase deref cases) = do
     matches <- mapM toMatch cases
     return $ CaseE exp_ matches
   where
-    readMay s =
-        case reads s of
-            (x, ""):_ -> Just x
-            _ -> Nothing
     toMatch :: (Binding, [Doc]) -> Q Match
     toMatch (idents, inside) = do
         (pat, extraScope) <- bindingPattern idents
@@ -344,9 +359,13 @@ data Env = Env
 hamletFromString :: Q HamletRules -> HamletSettings -> String -> Q Exp
 hamletFromString qhr set s = do
     hr <- qhr
+    hrWithEnv hr $ \env -> docsToExp env hr [] $ docFromString set s
+
+docFromString :: HamletSettings -> String -> [Doc]
+docFromString set s =
     case parseDoc set s of
         Error s' -> error s'
-        Ok (_mnl, d) -> hrWithEnv hr $ \env -> docsToExp env hr [] d
+        Ok (_, d) -> d
 
 hamletFileWithSettings :: Q HamletRules -> HamletSettings -> FilePath -> Q Exp
 hamletFileWithSettings qhr set fp = do
@@ -359,6 +378,14 @@ hamletFileWithSettings qhr set fp = do
 hamletFile :: FilePath -> Q Exp
 hamletFile = hamletFileWithSettings hamletRules defaultHamletSettings
 
+hamletFileReload :: FilePath -> Q Exp
+hamletFileReload = hamletFileReloadWithSettings runtimeRules defaultHamletSettings
+  where runtimeRules = HamletRuntimeRules { hrrI18n = False }
+
+ihamletFileReload :: FilePath -> Q Exp
+ihamletFileReload = hamletFileReloadWithSettings runtimeRules defaultHamletSettings
+  where runtimeRules = HamletRuntimeRules { hrrI18n = True }
+
 xhamletFile :: FilePath -> Q Exp
 xhamletFile = hamletFileWithSettings hamletRules xhtmlHamletSettings
 
@@ -393,3 +420,168 @@ condH bms mm = fromMaybe (return ()) $ lookup True bms `mplus` mm
 -- Otherwise, runs the third argument, if available.
 maybeH :: Monad m => Maybe v -> (v -> m ()) -> Maybe (m ()) -> m ()
 maybeH mv f mm = fromMaybe (return ()) $ fmap f mv `mplus` mm
+
+
+type MTime = UTCTime
+data VarType = VTPlain | VTUrl | VTUrlParam | VTMixin | VTMsg | VTAttrs
+
+type QueryParameters = [(Text, Text)]
+type RenderUrl url   = (url -> QueryParameters -> Text)
+type Shakespeare url = RenderUrl url -> Html
+data VarExp msg url  = EPlain Html
+                     | EUrl url
+                     | EUrlParam (url, QueryParameters)
+                     | EMixin (HtmlUrl url)
+                     | EMixinI18n (HtmlUrlI18n msg url)
+                     | EMsg msg
+
+getVars :: Content -> [(Deref, VarType)]
+getVars ContentRaw{}     = []
+getVars (ContentVar d)   = [(d, VTPlain)]
+getVars (ContentUrl False d) = [(d, VTUrl)]
+getVars (ContentUrl True d) = [(d, VTUrlParam)]
+getVars (ContentEmbed d) = [(d, VTMixin)]
+getVars (ContentMsg d)   = [(d, VTMsg)]
+getVars (ContentAttrs d) = [(d, VTAttrs)]
+
+hamletUsedIdentifiers :: HamletSettings -> String -> [(Deref, VarType)]
+hamletUsedIdentifiers settings =
+    concatMap getVars . contentFromString settings
+
+
+data HamletRuntimeRules = HamletRuntimeRules {
+                            hrrI18n :: Bool
+                          }
+
+hamletFileReloadWithSettings :: HamletRuntimeRules
+                             -> HamletSettings -> FilePath -> Q Exp
+hamletFileReloadWithSettings hrr settings fp = do
+    s <- readFileQ fp
+    let b = hamletUsedIdentifiers settings s
+    c <- mapM vtToExp b
+    rt <- if hrrI18n hrr
+      then [|hamletRuntimeMsg settings fp|]
+      else [|hamletRuntime settings fp|]
+    return $ rt `AppE` ListE c
+  where
+    vtToExp :: (Deref, VarType) -> Q Exp
+    vtToExp (d, vt) = do
+        d' <- lift d
+        c' <- toExp vt
+        return $ TupE [d', c' `AppE` derefToExp [] d]
+      where
+        toExp = c
+          where
+            c :: VarType -> Q Exp
+            c VTAttrs = [|EPlain . attrsToHtml . toAttributes|]
+            c VTPlain = [|EPlain . toHtml|]
+            c VTUrl = [|EUrl|]
+            c VTUrlParam = [|EUrlParam|]
+            c VTMixin = [|\r -> EMixin $ \c -> r c|]
+            c VTMsg = [|EMsg|]
+
+-- move to Shakespeare.Base?
+readFileUtf8 :: FilePath -> IO String
+readFileUtf8 fp = fmap TL.unpack $ readUtf8File fp
+
+-- move to Shakespeare.Base?
+readFileQ :: FilePath -> Q String
+readFileQ fp = qRunIO $ readFileUtf8 fp
+
+{-# NOINLINE reloadMapRef #-}
+reloadMapRef :: IORef (M.Map FilePath (MTime, [Content]))
+reloadMapRef = unsafePerformIO $ newIORef M.empty
+
+lookupReloadMap :: FilePath -> IO (Maybe (MTime, [Content]))
+lookupReloadMap fp = do
+  reloads <- readIORef reloadMapRef
+  return $ M.lookup fp reloads
+
+insertReloadMap :: FilePath -> (MTime, [Content]) -> IO [Content]
+insertReloadMap fp (mt, content) = atomicModifyIORef reloadMapRef
+  (\reloadMap -> (M.insert fp (mt, content) reloadMap, content))
+
+contentFromString :: HamletSettings -> String -> [Content]
+contentFromString set = map justContent . docFromString set
+  where
+    unsupported msg = error $ "hamletFileReload does not support " ++ msg
+
+    justContent :: Doc -> Content
+    justContent (DocContent c) = c
+    justContent DocForall{} = unsupported "$forall"
+    justContent DocWith{} = unsupported "$with"
+    justContent DocMaybe{} = unsupported "$maybe"
+    justContent DocCase{} = unsupported "$case"
+    justContent DocCond{} = unsupported "attribute conditionals"
+
+
+hamletRuntime :: HamletSettings
+              -> FilePath
+              -> [(Deref, VarExp msg url)]
+              -> Shakespeare url
+hamletRuntime settings fp cd render = unsafePerformIO $ do
+    mtime <- qRunIO $ getModified $ decodeString fp
+    mdata <- lookupReloadMap fp
+    case mdata of
+      Just (lastMtime, lastContents) ->
+        if mtime == lastMtime then return $ go' lastContents
+          else fmap go' $ newContent mtime
+      Nothing -> fmap go' $ newContent mtime
+  where
+    newContent mtime = do
+        s <- readFileUtf8 fp
+        insertReloadMap fp (mtime, contentFromString settings s)
+
+    go' = mconcat . map (runtimeContentToHtml cd render (error "I18n embed IMPOSSIBLE") handleMsgEx)
+    handleMsgEx _ = error "i18n _{} encountered, but did not use ihamlet"
+
+type RuntimeVars msg url = [(Deref, VarExp msg url)]
+hamletRuntimeMsg :: HamletSettings
+              -> FilePath
+              -> RuntimeVars msg url
+              -> HtmlUrlI18n msg url
+hamletRuntimeMsg settings fp cd i18nRender render = unsafePerformIO $ do
+    mtime <- qRunIO $ getModified $ decodeString fp
+    mdata <- lookupReloadMap fp
+    case mdata of
+      Just (lastMtime, lastContents) ->
+        if mtime == lastMtime then return $ go' lastContents
+          else fmap go' $ newContent mtime
+      Nothing -> fmap go' $ newContent mtime
+  where
+    newContent mtime = do
+        s <- readFileUtf8 fp
+        insertReloadMap fp (mtime, contentFromString settings s)
+
+    go' = mconcat . map (runtimeContentToHtml cd render i18nRender handleMsg)
+    handleMsg d = case lookup d cd of
+            Just (EMsg s) -> i18nRender s
+            _ -> error $ show d ++ ": expected EMsg for ContentMsg"
+
+runtimeContentToHtml :: RuntimeVars msg url -> Render url -> Translate msg -> (Deref -> Html) -> Content -> Html
+runtimeContentToHtml cd render i18nRender handleMsg = go
+  where
+    go :: Content -> Html
+    go (ContentMsg d) = handleMsg d
+    go (ContentRaw s) = preEscapedToHtml s
+    go (ContentAttrs d) =
+        case lookup d cd of
+            Just (EPlain s) -> s
+            _ -> error $ show d ++ ": expected EPlain for ContentAttrs"
+    go (ContentVar d) =
+        case lookup d cd of
+            Just (EPlain s) -> s
+            _ -> error $ show d ++ ": expected EPlain for ContentVar"
+    go (ContentUrl False d) =
+        case lookup d cd of
+            Just (EUrl u) -> toHtml $ render u []
+            _ -> error $ show d ++ ": expected EUrl"
+    go (ContentUrl True d) =
+        case lookup d cd of
+            Just (EUrlParam (u, p)) ->
+                toHtml $ render u p
+            _ -> error $ show d ++ ": expected EUrlParam"
+    go (ContentEmbed d) = case lookup d cd of
+        Just (EMixin m) -> m render
+        Just (EMixinI18n m) -> m i18nRender render
+        _ -> error $ show d ++ ": expected EMixin"
